@@ -1,4 +1,4 @@
-// Copyright (C) 2010-2018 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2010-2023 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -44,6 +44,17 @@ namespace asiolink {
 /// This class is a wrapper for the ASIO \c ip::address class.
 class IOAddress {
 public:
+
+    /// \brief An \c IOAddress hash enabling the use in the unordered
+    /// STL containers.
+    struct Hash {
+        /// \brief A hashing operator.
+        ///
+        /// \param io_address an address to be hashed.
+        /// \return a hashing result.
+        size_t operator()(const IOAddress &io_address) const;
+    };
+
     ///
     /// \name Constructors and Destructor
     ///
@@ -187,52 +198,15 @@ public:
     /// \brief Checks if one address is smaller than the other
     ///
     /// \param other Address to compare against.
-    ///
-    /// \return true if this address is smaller than the other address.
-    ///
-    /// It is useful for comparing which address is bigger.
-    /// Operations within one protocol family are obvious.
-    /// Comparisons between v4 and v6 will always return v4
-    /// being smaller. This follows boost::boost::asio::ip implementation
-    bool lessThan(const IOAddress& other) const {
-        if (this->getFamily() == other.getFamily()) {
-            if (this->getFamily() == AF_INET6) {
-                return (this->asio_address_.to_v6() < other.asio_address_.to_v6());
-            } else {
-                return (this->asio_address_.to_v4() < other.asio_address_.to_v4());
-            }
-        }
-        return (this->getFamily() < other.getFamily());
-    }
-
-    /// \brief Checks if one address is smaller or equal than the other
-    ///
-    /// \param other Address to compare against.
-    ///
-    /// \return true if this address is smaller than the other address.
-    bool smallerEqual(const IOAddress& other) const {
-        if (equals(other)) {
-            return (true);
-        }
-        return (lessThan(other));
-    }
-
-    /// \brief Checks if one address is smaller than the other
-    ///
-    /// \param other Address to compare against.
-    ///
-    /// See \ref lessThan method for details.
     bool operator<(const IOAddress& other) const {
-        return (lessThan(other));
+        return (asio_address_ < other.asio_address_);
     }
 
     /// \brief Checks if one address is smaller or equal than the other
     ///
     /// \param other Address to compare against.
-    ///
-    /// See \ref smallerEqual method for details.
     bool operator<=(const IOAddress& other) const {
-        return (smallerEqual(other));
+        return (asio_address_ <= other.asio_address_);
     }
 
     /// \brief Compare addresses for inequality
@@ -339,6 +313,17 @@ private:
 std::ostream&
 operator<<(std::ostream& os, const IOAddress& address);
 
+/// \brief Hash the IOAddress.
+///
+/// This method allows boost multi-index hashed indexes on IOAddresses.
+/// It follows the requirement with equality: if two addresses are equal
+/// their hashes are equal, if two addresses are not equal their hashes
+/// are almost surely not equal.
+///
+/// \param address A \c IOAddress to hash.
+/// \return The hash of the IOAddress.
+size_t hash_value(const IOAddress& address);
+
 } // namespace asiolink
 } // namespace isc
 #endif // IO_ADDRESS_H
